﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using Rhino.Mocks;
using System.Collections.Specialized;

namespace Tests
{
  public static class MvcMockHelpers
  {
    public static IHttpContext DynamicIHttpContext(this MockRepository mocks)
    {
      IHttpContext context = mocks.DynamicMock<IHttpContext>();
      IHttpRequest request = mocks.DynamicMock<IHttpRequest>();
      IHttpResponse response = mocks.DynamicMock<IHttpResponse>();
      IHttpSessionState session = mocks.DynamicMock<IHttpSessionState>();
      IHttpServerUtility server = mocks.DynamicMock<IHttpServerUtility>();

      SetupResult.For(context.Request).Return(request);
      SetupResult.For(context.Response).Return(response);
      SetupResult.For(context.Session).Return(session);
      SetupResult.For(context.Server).Return(server);

      mocks.Replay(context);
      return context;
    }

    public static IHttpContext DynamicIHttpContext(this MockRepository mocks, string url)
    {
      IHttpContext context = DynamicIHttpContext(mocks);
      context.Request.SetMockedRequestUrl(url);
      return context;
    }

    static string GetUrlFileName(string url)
    {
      if (url.Contains("?"))
        return url.Substring(0, url.IndexOf("?"));
      else
        return url;
    }

    static NameValueCollection GetQueryStringParameters(string url)
    {
      if (url.Contains("?"))
      {
        NameValueCollection parameters = new NameValueCollection();

        string[] parts = url.Split("?".ToCharArray());
        string[] keys = parts[1].Split("&".ToCharArray());

        foreach (string key in keys)
        {
          string[] part = key.Split("=".ToCharArray());
          parameters.Add(part[0], part[1]);
        }

        return parameters;
      }
      else
      {
        return null;
      }
    }

    public static void SetFakeHttpMethod(this IHttpRequest request, string httpMethod)
    {
      SetupResult.For(request.HttpMethod).Return(httpMethod);
    }

    public static void SetMockedRequestUrl(this IHttpRequest request, string url)
    {
      if (url == null)
        throw new ArgumentNullException("url");

      if (!url.StartsWith("~/"))
        throw new ArgumentException("Sorry, we expect a virtual url starting with \"~/\".");

      SetupResult.For(request.QueryString).Return(GetQueryStringParameters(url));
      SetupResult.For(request.AppRelativeCurrentExecutionFilePath).Return(GetUrlFileName(url));
      SetupResult.For(request.PathInfo).Return(string.Empty);
    }
  }
}
